﻿///<reference path="http://extjs.cachefly.net/ext-3.2.1/adapter/ext/ext-base-debug.js" />
///<reference path="http://extjs.cachefly.net/ext-3.2.1/ext-all-debug.js" />
Ext.ns('Ext.ux.mvc');

Ext.ux.mvc.Validation = Ext.apply(new Ext.util.Observable(), {
	forms: undefined,
	fieldErrCss: 'field-validation-error',
	fieldValidCss: 'field-validation-valid',
	inputErrCss: 'input-validation-error',
	inputValidCss: 'input-validation-valid',
	summaryErrCls: 'validation-summary-errors',
	summaryValidCls: 'validation-summary-valid',
	init: function() {
		this.addEvents('beforeregister', 'afterregister', 'beforevalidate', 'aftervalidate');
		var allFormOpts = window.mvcClientValidationMetadata;
		if (allFormOpts) {
			Ext.each(allFormOpts, this.registerForm, this);
		}
	},
	registerForm: function(formOpts, index, allFormOpts) {
		if (this.fireEvent('beforeregister', formOpts) === false) {
			return;
		}
		if (!this.forms) {
			this.forms = {};
		}
		var formId = formOpts.FormId;
		this.forms[formId] = formOpts;
		formOpts.el = Ext.get(formId);
		formOpts.el.dom.onsubmit = this.validateForm.createDelegate(this, [formId]);
		Ext.each(formOpts.Fields, this.registerField, this);
		this.fireEvent('afterregister', formOpts);
	},
	registerField: function(fieldOpts, index, formOpts) {
		var fieldO = fieldOpts;
		var formO = formOpts;
		Ext.fly(fieldOpts.FieldName).on('blur', function() {
			this.validateField(fieldOpts, formOpts);
		}, this);
	},
	validateForm: function(formId) {
		var formOpts = this.forms[formId];
		if (Ext.isEmpty(formOpts) || this.fireEvent('beforevalidate', formOpts) === false) {
			return true;
		}
		delete formOpts.errors;
		formOpts.errors = [];
		for (var i = 0; i < formOpts.Fields.length; i++) {
			this.validateField(formOpts.Fields[i], formOpts);
		}
		var formIsValid = formOpts.errors.length == 0;
		this.updateValidationSummary(formOpts.ValidationSummaryId, formOpts.errors, formOpts.ReplaceValidationSummary);
		return formIsValid;
	},
	validateField: function(fieldOpts, formOpts) {
		var val = Ext.get(fieldOpts.FieldName).getValue();
		var rules = fieldOpts.ValidationRules;
		if (!formOpts.errors) {
			formOpts.errors = [];
		}
		for (var i = 0; i < rules.length; i++) {
			var validator = Ext.ux.mvc.VTypes[rules[i].ValidationType];
			if (!Ext.isEmpty(validator)) {
				if (validator(val, rules[i].ValidationParameters) === false) {
					formOpts.errors.push(rules[i].ErrorMessage);
					this.updateValidationMessage({
						fieldId: fieldOpts.FieldName,
						msgId: fieldOpts.ValidationMessageId,
						msg: rules[i].ErrorMessage,
						replace: fieldOpts.ReplaceValidationMessageContents
					});
					return false;
					break;
				}
				else {
					this.updateValidationMessage({
						fieldId: fieldOpts.FieldName,
						msgId: fieldOpts.ValidationMessageId,
						msg: '',
						replace: fieldOpts.ReplaceValidationMessageContents
					});
				}
			}
			else {
				alert('客户端验证函数 ' + rules[i].ValidationType + ' 未实现！');
			}
		}
	},
	updateValidationMessage: function(opts) {
		if (opts.replace) {
			Ext.fly(opts.msgId).update(opts.msg);
		}
		if (Ext.isEmpty(opts.msg)) {
			Ext.fly(opts.msgId).removeClass(this.fieldErrCss).addClass(this.fieldValidCss);
			Ext.fly(opts.fieldId).removeClass(this.inputErrCss).addClass(this.inputValidCss);
		}
		else {
			Ext.fly(opts.msgId).removeClass(this.fieldValidCss).addClass(this.fieldErrCss);
			Ext.fly(opts.fieldId).removeClass(this.inputValidCss).addClass(this.inputErrCss);
		}
	},
	updateValidationSummary: function(summaryId, errors, replace) {
		if (Ext.isEmpty(summaryId)) {
			return;
		}
		if (replace) {
			var html = '';
			Ext.each(errors, function(err) {
				html += '<li>' + err + '</li>';
			});
			Ext.fly(summaryId).update(html);
		}
		if (errors.length == 0) {
			Ext.fly(summaryId).removeClass(this.summaryErrCls).addClass(this.summaryValidCls);
		}
		else {
			Ext.fly(summaryId).removeClass(this.summaryValidCls).addClass(this.summaryErrCls);
		}
	}
});

Ext.onReady(Ext.ux.mvc.Validation.init, Ext.ux.mvc.Validation);

Ext.ux.mvc.VTypes = function() {
	var alpha = /^[a-zA-Z_]+$/,
		 alphanum = /^[a-zA-Z0-9_]+$/,
		 email = /^(\w+)([\-+.][\w]+)*@(\w[\-\w]*\.){1,5}([A-Za-z]){2,6}$/,
		 url = /(((^https?)|(^ftp)):\/\/([\-\w]+\.)+\w{2,3}(\/[%\-\w]+(\.\w{2,})?)*(([\w\-\.\?\\\/+@&#;`~=%!]*)(\.\w{2,})?)*\/?)/i;
	return {
		required: function(v, param) {
			return !(Ext.isEmpty(v, false));
		},
		regularExpression: function(v, param) {
			var r = new RegExp(param.pattern);
			return r.test(v);
		},
		number: function(v, param) {
			return alphanum.test(v);
		},
		range: function(v, param) {
			return v >= param.minimum && v <= param.maximum;
		},
		stringLength: function(v, param) {
			if (Ext.isEmpty(v)) {
				return false;
			}
			return v.length >= param.minimumLength && v.length <= param.maximumLength;
		}
	}
}();