using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Web.Mvc;

namespace ExtMvcValidation.Models {

	public class RegisterModel {

		[Required(ErrorMessage = "ûд")]
		[DisplayName("û")]
		public string UserName {
			get;
			set;
		}

		[Required(ErrorMessage = "д")]
		[DisplayName("룺")]
		[DataType(DataType.Password)]
		public string Password {
			get;
			set;
		}

		[Required(ErrorMessage = "д")]
		[RegularExpression("", ErrorMessage = "ʼʽȷ")]
		[DisplayName("䣺")]
		public string Email {
			get;
			set;
		}

		[Range(0, 100, ErrorMessage = "1~100֮䣡")]
		[DisplayName("䣺")]
		[DefaultValue(20)]
		public int Age {
			get;
			set;
		}

		[Custom]
		[DisplayName(": Hello,world!")]
		public string CustomProperty {
			get;
			set;
		}
	}

	[AttributeUsage(AttributeTargets.Property, AllowMultiple = false)]
	public class CustomAttribute : ValidationAttribute {

		public override bool IsValid(object value) {
			var val = value as string;
			if (string.IsNullOrEmpty(val)) {
				return false;
			}
			return val.Equals("Hello,world!", StringComparison.OrdinalIgnoreCase);
		}
	}

	public class CustomValidator : DataAnnotationsModelValidator<CustomAttribute> {

		public CustomValidator(ModelMetadata metadata, ControllerContext context, CustomAttribute attribute)
			: base(metadata, context, attribute) {
		}

		public override IEnumerable<ModelClientValidationRule> GetClientValidationRules() {
			return new[] {
				new ModelClientValidationRule {
					ErrorMessage = ": Hello,world!",
					ValidationType = "custom"
				}
			};
		}
	}
}